----------------------------------------------------------------------------
----------------------------------------------------------------------------
--  The Free IP Project
--  VHDL Free-6502 Core
--  (c) 1999, The Free IP Project and David Kessner
--
--
--  FREE IP GENERAL PUBLIC LICENSE
--  TERMS AND CONDITIONS FOR USE, COPYING, DISTRIBUTION, AND MODIFICATION
--
--  1.  You may copy and distribute verbatim copies of this core, as long
--      as this file, and the other associated files, remain intact and
--      unmodified.  Modifications are outlined below.  Also, see the
--      import/export warning above for further restrictions on
--      distribution.
--  2.  You may use this core in any way, be it academic, commercial, or
--      military.  Modified or not.
--  3.  Distribution of this core must be free of charge.  Charging is
--      allowed only for value added services.  Value added services
--      would include copying fees, modifications, customizations, and
--      inclusion in other products.
--  4.  If a modified source code is distributed, the original unmodified
--      source code must also be included (or a link to the Free IP web
--      site).  In the modified source code there must be clear
--      identification of the modified version.
--  5.  Visit the Free IP web site for additional information.
--      http://www.free-ip.com
--
----------------------------------------------------------------------------
----------------------------------------------------------------------------
--
--  This is a ROM to test the BCD mode support added by Sprow 22-Dec-2000
--
--  .binmode_test1
--  CLD
--  LDA#&59
--  CLC
--  ADC#1
--  STAresult:\Expect &5A on the bus
--  :
--  .binmode_test2
--  ADC#1
--  STAresult:\Expect &5B on the bus
--  :
--  .bcdmode_test3
--  SED
--  LDA#&59
--  CLC
--  ADC#1
--  STAresult:\Expect &60 on the bus
--  :
--  .bcdmode_test3
--  ADC#1
--  STAresult:\Expect &61 on the bus
--  :
--  .bcdmode_test4
--  LDA#&5A
--  CLC
--  ADC#&2
--  STAresult:\Expect &5A to be clipped to &60,so the result is &62
--  :
--  .bcdmode_test5
--  LDA#&5A
--  CLC
--  ADC#&B
--  STAresult:\Expect &5A to be clipped to &60,but the ADC argument doesn't get clipped so the result is &6B!!
--  :
--  .bcdmode_test6
--  LDA#&2A
--  SEC
--  ADC#&2
--  STAresult:\Expect &2A to be clipped to &30,but as carry is set the result is &33
--  :
--  .bcdmode_test7
--  LDA#&2A
--  SEC
--  ADC#&F
--  STAresult:\Expect &2A to be clipped to &30,but the ADC argument doesn't get clipped,but C=1 so the result is &30!!
--  JMPbcdmode_test1
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;

package test_suite is
  component test_rom
    port (addr	:in std_logic_vector (15 downto 0);
          data  :out std_logic_vector (7 downto 0)
         );
  end component;
end test_suite;


------------------------------------------------------------------------------
------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.test_suite.all;

entity test_rom is
    port (addr	:in std_logic_vector (15 downto 0);
          data  :out std_logic_vector (7 downto 0)
         );
end test_rom;

architecture arch_test_rom of test_rom is
begin
  process (addr)
  begin
    case addr(15 downto 0) is
      --addrbits 8
      --databits 8
      --varname data
      --little-endian
      --start_of_rom
      when "0000110000000000" =>  data <= "11011000";  -- 0 = D8
      when "0000110000000001" =>  data <= "10101001";  -- 1 = A9
      when "0000110000000010" =>  data <= "01011001";  -- 2 = 59
      when "0000110000000011" =>  data <= "00011000";  -- 3 = 18
      when "0000110000000100" =>  data <= "01101001";  -- 4 = 69
      when "0000110000000101" =>  data <= "00000001";  -- 5 = 1
      when "0000110000000110" =>  data <= "10001101";  -- 6 = 8D
      when "0000110000000111" =>  data <= "00111111";  -- 7 = 3F
      when "0000110000001000" =>  data <= "00001100";  -- 8 = C
      when "0000110000001001" =>  data <= "01101001";  -- 9 = 69
      when "0000110000001010" =>  data <= "00000001";  -- A = 1
      when "0000110000001011" =>  data <= "10001101";  -- B = 8D
      when "0000110000001100" =>  data <= "00111111";  -- C = 3F
      when "0000110000001101" =>  data <= "00001100";  -- D = C
      when "0000110000001110" =>  data <= "11111000";  -- E = F8
      when "0000110000001111" =>  data <= "10101001";  -- F = A9
      when "0000110000010000" =>  data <= "01011001";  -- 10 = 59
      when "0000110000010001" =>  data <= "00011000";  -- 11 = 18
      when "0000110000010010" =>  data <= "01101001";  -- 12 = 69
      when "0000110000010011" =>  data <= "00000001";  -- 13 = 1
      when "0000110000010100" =>  data <= "10001101";  -- 14 = 8D
      when "0000110000010101" =>  data <= "00111111";  -- 15 = 3F
      when "0000110000010110" =>  data <= "00001100";  -- 16 = C
      when "0000110000010111" =>  data <= "01101001";  -- 17 = 69
      when "0000110000011000" =>  data <= "00000001";  -- 18 = 1
      when "0000110000011001" =>  data <= "10001101";  -- 19 = 8D
      when "0000110000011010" =>  data <= "00111111";  -- 1A = 3F
      when "0000110000011011" =>  data <= "00001100";  -- 1B = C
      when "0000110000011100" =>  data <= "10101001";  -- 1C = A9
      when "0000110000011101" =>  data <= "01011010";  -- 1D = 5A
      when "0000110000011110" =>  data <= "00011000";  -- 1E = 18
      when "0000110000011111" =>  data <= "01101001";  -- 1F = 69
      when "0000110000100000" =>  data <= "00000010";  -- 20 = 2
      when "0000110000100001" =>  data <= "10001101";  -- 21 = 8D
      when "0000110000100010" =>  data <= "00111111";  -- 22 = 3F
      when "0000110000100011" =>  data <= "00001100";  -- 23 = C
      when "0000110000100100" =>  data <= "10101001";  -- 24 = A9
      when "0000110000100101" =>  data <= "01011010";  -- 25 = 5A
      when "0000110000100110" =>  data <= "00011000";  -- 26 = 18
      when "0000110000100111" =>  data <= "01101001";  -- 27 = 69
      when "0000110000101000" =>  data <= "00001011";  -- 28 = B
      when "0000110000101001" =>  data <= "10001101";  -- 29 = 8D
      when "0000110000101010" =>  data <= "00111111";  -- 2A = 3F
      when "0000110000101011" =>  data <= "00001100";  -- 2B = C
      when "0000110000101100" =>  data <= "10101001";  -- 2C = A9
      when "0000110000101101" =>  data <= "00101010";  -- 2D = 2A
      when "0000110000101110" =>  data <= "00111000";  -- 2E = 38
      when "0000110000101111" =>  data <= "01101001";  -- 2F = 69
      when "0000110000110000" =>  data <= "00000010";  -- 30 = 2
      when "0000110000110001" =>  data <= "10001101";  -- 31 = 8D
      when "0000110000110010" =>  data <= "00111111";  -- 32 = 3F
      when "0000110000110011" =>  data <= "00001100";  -- 33 = C
      when "0000110000110100" =>  data <= "10101001";  -- 34 = A9
      when "0000110000110101" =>  data <= "00101010";  -- 35 = 2A
      when "0000110000110110" =>  data <= "00111000";  -- 36 = 38
      when "0000110000110111" =>  data <= "01101001";  -- 37 = 69
      when "0000110000111000" =>  data <= "00001111";  -- 38 = F
      when "0000110000111001" =>  data <= "10001101";  -- 39 = 8D
      when "0000110000111010" =>  data <= "00111111";  -- 3A = 3F
      when "0000110000111011" =>  data <= "00001100";  -- 3B = C
      when "0000110000111100" =>  data <= "01001100";  -- 3C = 4C
      when "0000110000111101" =>  data <= "00000000";  -- 3D = 0
      when "0000110000111110" =>  data <= "00001100";  -- 3E = C
      when "0000110000111111" =>  data <= "00000000";  -- 3F = 0
      --vectors
      when "1111111100000000" =>  data <= "01000000";  -- FF00 = RTI instruction
      when "1111111111111010" =>  data <= "00000000";  -- FFFA
      when "1111111111111011" =>  data <= "11111111";  -- FFFB = FF00 -> nmi handler
      when "1111111111111100" =>  data <= "00000000";  -- FFFC
      when "1111111111111101" =>  data <= "00001100";  -- FFFD = 0C00 -> reset code
      when "1111111111111110" =>  data <= "00000000";  -- FFFE
      when "1111111111111111" =>  data <= "11111111";  -- FFFF = FF00 -> irq handler
      --end_of_rom
      when others =>  data <= "00000000";
    end case;
  end process;

end arch_test_rom;

------------------------------------------------------------------------------
------------------------------------------------------------------------------

